require 'singleton'
require 'framework/mixins/notifier'

module Mondrian
	class Debugger
		include Singleton
		include Notifier
		event :breakpoint_added, :breakpoint_removed
		attr_reader :breakpoints
		
		def initialize
			@breakpoints = []
		end
		
		def add_breakpoint(bp)
			return if @breakpoints.include? bp
			@breakpoints << bp
			notify_breakpoint_added(bp)
		end
		
		def remove_breakpoint(bp)
			notify_breakpoint_removed(bp) if @breakpoints.delete(bp)			
		end				
		
		def toggle_breakpoint(bp)			
			@breakpoints.include?(bp) ? remove_breakpoint(bp) : add_breakpoint(bp)
		end
		
		def replace_breakpoint(old, new)
			remove_breakpoint(old)
			add_breakpoint(new)
		end
	end
	
	Breakpoint = Struct.new(:file_path, :line)
	class Breakpoint
		def inspect
			"#{file_path}:#{line+1}"
		end
	end		
end
			
			