
module Mondrian

  class SymbolListItem < FXListItem 
    def initialize(element, file)
      @element = element
      name = element.name
      name += "(#{element.parameters.join(",")})" if element.is_a?(RubyMethod)
      name += " [in #{[file.name, element.parent_qname].compact.join("/")}]" if not element.is_a?(RubyFile)    
      
      super(name)
      self.icon = IconManager.instance.icon(element.class)            
    end

    def reveal(launcher)
      @element.reveal(launcher)
    end
  end  

  class OpenSymbolDialog < FXDialogBox
    def initialize(view_launcher, file=nil)
			title = file ? "Open Symbol in File" : "Open Symbol"
      super(SessionManager.instance.main_window, title, DECOR_TITLE | DECOR_BORDER | DECOR_CLOSE | DECOR_RESIZE, 0, 0, 0, 0)
      self.miniIcon = SessionManager.instance.main_window.miniIcon
			@view_launcher = view_launcher
      @file = file
      
      frame = FXVerticalFrame.new(self, FRAME_NONE | LAYOUT_FILL_X | LAYOUT_FILL_Y)            
      
      top_frame = FXVerticalFrame.new(frame, FRAME_NONE | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)      
      find_frame = FXHorizontalFrame.new(top_frame, FRAME_NONE | LAYOUT_FILL_X, 0, 0, 0, 0, 0, 0, 0, 0)
      
      @search_field = FXTextField.new(find_frame, 25, self, ID_ACCEPT, TEXTFIELD_NORMAL | TEXTFIELD_ENTER_ONLY | LAYOUT_FILL_X)
      @search_field.connect(SEL_CHANGED, method(:search_text_changed))      
      @search_field.numColumns = 65
      @search_field.connect(SEL_COMMAND, method(:on_double_clicked))

      symbols_frame = FXVerticalFrame.new(top_frame, FRAME_NONE | LAYOUT_FILL_X | LAYOUT_FILL_Y, 0, 0, 0, 0, 0, 0, 0, 0)
            
      @symbol_list = FXList.new(symbols_frame, nil, 0, FRAME_NORMAL | LAYOUT_FIX_WIDTH | LIST_BROWSESELECT | LAYOUT_FILL_X | LAYOUT_FILL_Y)
      @symbol_list.setNumVisible(25)
      @symbol_list.connect(SEL_DOUBLECLICKED, method(:on_double_clicked))      
              
      button_frame = FXHorizontalFrame.new(top_frame, FRAME_NONE | LAYOUT_RIGHT | PACK_UNIFORM_WIDTH, 0, 0, 0, 0, 0, 0, 10, 0)
      button_dims = [0, 0, 0, 0, 15, 15]
      FXButton.new(button_frame, "&Open", nil, self, ID_ACCEPT, BUTTON_NORMAL | BUTTON_DEFAULT | BUTTON_INITIAL, *button_dims)
      FXButton.new(button_frame, "&Cancel", nil, self, ID_CANCEL, BUTTON_NORMAL, *button_dims)
            
      self.connect(SEL_CONFIGURE) { 
        @symbol_list.width = @search_field.width
        @search_field.setFocus
      }
      populate_items      
    end
    
    def OpenSymbolDialog.browse_to(viewer_launcher, sym)
      dlg = OpenSymbolDialog.new(viewer_launcher)
      dlg.symbol = "^#{sym}$"
      if dlg.multiple_matches?
        dlg.execute
      else
        dlg.reveal_selected
      end
    end
    
    def symbol=(sym)
      @search_field.text = sym
      populate_items
    end
    
    def multiple_matches?
      return @symbol_list.numItems > 1
    end
    
    def collect_items(&proc)      
      @file ? @file.children.each { |item| recursive_collect(item, @file, &proc) } :
        FileManager.instance.each(RubyFile) { |item| recursive_collect(item, item, &proc) }
    end
    
    def recursive_collect(element, file, &proc)      
      if element.respond_to?(:reveal)
        yield element, file if survives_filter?(element)
        element.children.each do |child|
          recursive_collect(child, file, &proc)
        end
      end      
    end
    
    def populate_items
      @symbol_list.clearItems
      collect_items { |element, file| @symbol_list.appendItem(SymbolListItem.new(element, file)) }
      @symbol_list.sortItems
      if @symbol_list.numItems > 0
        @symbol_list.selectItem(0)
        @symbol_list.setCurrentItem(0) 
      end
    end
        
    def survives_filter?(element)
      match_text = @search_field.text.strip.upcase
      return true if match_text.length == 0
      begin
        element.name.upcase =~ Regexp.new(match_text)
      rescue
        true
      end
    end
    
    def search_text_changed(sender, message, data)
      getApp.removeTimeout(@timer) if @timer
      @timer = getApp.addTimeout(200) { |sender, sel, data|
        populate_items
      }      
    end
    
    def on_double_clicked(sender, message, index)
      getApp.stopModal(1)
      hide
    end
        
    def execute      
      reveal_selected if (result = super) == 1
      result
    end
    
    def reveal_selected
      @symbol_list.getItem(@symbol_list.currentItem).reveal(@view_launcher) if (@symbol_list.currentItem >= 0)        
    end
  end  
  
  class RubyFile   
    def parent_qname
      nil
    end
    
    def qname
      nil
    end
  end
  
[RubyFile, SourceParser::RubyClass, SourceParser::RubyMethod, SourceParser::RubyModule].each do |c| 
  c.class_eval do
    def reveal(launcher)      
      launcher.launch_viewer(nil, self)
    end     
  end
end

  module SourceParser
    class Molecule      
      def parent_qname
        parent ? parent.qname : nil
      end
           
      def qname
        return nil if root?
        return parent.qname + "::" + name if parent.qname
        name
      end
    end
  end
end