# texteditor.rb
#
# Contributors:: Oliver Smith


require 'framework/mixins/editable'


module Mondrian

    # A proto-GUI class for editing a +TokenString+.

    class TextEditor
        include Editable
        
        event :text_inserted, :text_deleted, :text_lexed
        
        attr_reader :tokenstring
        
        def initialize(tokenstring)
            @tokenstring = tokenstring
        end
        
        
        # Inserts +text+ at +position+ and calls the appropriate events.
        
        def insert_text(text, position)
            affected_range = @tokenstring.insert_text(text, position)
            
            notify_text_inserted text, position
            notify_text_lexed affected_range
        end


        # Deletes +length+ characters at +position+ and calls the appropriate events.

        def delete_text(length, position)
            affected_range = @tokenstring.delete_text(length, position)
            
            notify_text_deleted length, position
            notify_text_lexed affected_range
        end


        # Allows us to treat this class like a +TokenString+.
    
        def method_missing(method_symbol, *args, &block)
            @tokenstring.send(method_symbol, *args, &block) 
        end

    end
    
end