# workspace.rb
#
# Contributors:: Oliver Smith


require 'yaml'

require 'framework/common/utilities'
require 'framework/project'

include Mondrian


module Mondrian

    # A class for managing a workspace directory.

    class Workspace

        attr_reader :path
    
        def self.create(path)
            Dir.mkdir(path) unless File.exists?(path)
            
            self.new(path)
        end


        def initialize(path)
            raise Errno::ENOENT unless File.directory?(path)
        
            @path = File.expand_path(path)
        end


        # Returns whether +abs_path+ is a project folder in this workspace.

        def project_folder?(abs_path)
            raise ArgumentError, "Path (#{abs_path}) is not absolute!" unless is_absolute? abs_path

            return Project.project_folder?(abs_path) && is_subpath?(abs_path, @path)
        end


        # Returns the absolute names of project folders.
        
        def project_folders
            folders = Dir.new(@path).entries.collect { |entry| File.join(@path, entry) }.select { |entry| project_folder? entry }
            
            folders.sort { |a, b| File.basename(a).downcase <=> File.basename(b).downcase }
        end
    
    end
    
end